package cn.estsh.i3plus.ext.mes.pcn.apiservice.util;

import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

@Component
public class MesReentrantLockUtil {

    private final static Map<String, ReentrantLock> lockMap = new ConcurrentHashMap<>();

    public static Boolean tryLock(List<ReentrantLock> acquiredLocks, List<String> keyList, String prefix, Long milliseconds) throws InterruptedException {

        Collections.sort(keyList);

        long remaining = TimeUnit.MILLISECONDS.toNanos(milliseconds);
        long endTime = System.nanoTime() + remaining;

        for (String key : keyList) {

            if (StringUtils.isEmpty(key)) continue;

            ReentrantLock lock = lockMap.computeIfAbsent(prefix + key, o -> new ReentrantLock());

            if (!lock.tryLock(remaining, TimeUnit.NANOSECONDS)) return false;

            acquiredLocks.add(lock);

            remaining = endTime - System.nanoTime();
        }

        return true;

    }

    public static void unLockAll(List<ReentrantLock> acquiredLocks) {
        for (int i = acquiredLocks.size() - 1; i >= 0; i --) acquiredLocks.get(i).unlock();
    }

}
